clear all
indir='C:\Mitochondrial morphology20220929\'; %  the input directory
outdir='C:\Mitochondrial morphology20220929\out\'% the output directory
run('C:\Program Files\DIPimage 2.9\dipstart.m'); %DipImage University of Delft

pxlsz=0.3256513; %the pixel size 
pxlsZ=0.350; % physical size
Vxlsz=pxlsz^2*pxlsZ; %calculates voxel size
MitoCh=1;% first chanel of images is imaging mitochondria
Mitothr= 2.6353e+03;% empritically determined TMRM threshold 
maxproj=1;
Calcthr=1.6695e+03;% empritically determined Calcein threshold 
Nuclthr=3.1353e+03;% empritically determined Hoechst threshold 


if ~exist(outdir,'dir'), mkdir(outdir); end% makes the ourput folder

cd(indir);
files = dir('*C1*.tif'); %takes the first channnel for TMRM

[FileNr,~]=size(files);
C = cell(FileNr,26);


for p=1:FileNr

name = files(p).name;
prename=[name(1:end-10)];
calc = [prename,'3_cmle.tif'];% takes the third channnel for calcein
nucl = [prename,'2_cmle.tif']; % takes the second channnel for nucleus
readerMito = bfGetReader(name);
readerCalc = bfGetReader(calc);
readerNucl = bfGetReader(nucl);

ImstackMeta = readerMito.getMetadataStore();
ImstackSizeX = ImstackMeta.getPixelsSizeX(0).getValue(); % image width, pixels
ImstackSizeY = ImstackMeta.getPixelsSizeY(0).getValue(); % image height, pixels
ImstackSizeZ = ImstackMeta.getPixelsSizeZ(0).getValue(); % image height, pixels  

MitoRAW=uint16(zeros(ImstackSizeY,ImstackSizeX,ImstackSizeZ+2)); % import the mitoraw data
CalcRAW=uint16(zeros(ImstackSizeY,ImstackSizeX,ImstackSizeZ+2)); % import the Calcraw data
NuclRAW=uint16(zeros(ImstackSizeY,ImstackSizeX,ImstackSizeZ+2));% import the nucraw data


for z=1:ImstackSizeZ 
MitoRAW(:,:,z+1) = bfGetPlane(readerMito, z);
CalcRAW(:,:,z+1) = bfGetPlane(readerCalc, z);
NuclRAW(:,:,z+1) = bfGetPlane(readerNucl, z);

    end
calcM3D=medfilt3(CalcRAW,[5 5 5]);  %get the median data of 5 pixels in 3D 
[calcmask,~] = threshold(calcM3D,'fixed',Calcthr); %creates mask for calcein channel  
filledmask=calcmask;

for z=0:(ImstackSizeZ+1) 
BinImage=logical(calcmask(:,:,z)); %creates binary image of calcein mask
filledmask(:,:,z)= imfill(BinImage,'holes'); %filled the holles
end
 
Calclab=label(filledmask,2,200,0); % creates clacein label
CalcMsr=measure(Calclab,CalcRAW,({'Size'}),[],2,200,0);  %measures calcein channel
[calctest,~]=sort(CalcMsr.Size,'descend'); %sort cell size 
cellsize=CalcMsr.Size; %calculate cell size
cellvolume=cellsize*Vxlsz; %calculate cell volume

MitoM3D=medfilt3(MitoRAW);   %get the median data in 3D for mito
thf=[0 0 -1 -1 -1 0 0; 0 -1 -1 -1 -1 -1 0; -1 -1 3 3 3 -1 -1; -1 -1 3 4 3 -1 -1; -1 -1 3 3 3 -1 -1; 0 -1 -1 -1 -1 -1 0; 0 0 -1 -1 -1 0 0]; % does the top-hat filter
THF=imfilter(MitoM3D,thf,'conv');
[Mitomask,~] = threshold(THF,'fixed',Mitothr);%creates mask for mitochondia
realMitomask=Mitomask; 
Mitolab=label(logical(realMitomask),2,10,0); %creates mito label

NuclM3D=medfilt3(NuclRAW); %get the median data in 3D for nucleus
[Nuclmask,~] = threshold(NuclM3D,'fixed',Nuclthr);%creates nucleus mask 
Nuc_dil = dilation(Nuclmask,7,'elliptic');
Nuc_close = bclosing(Nuc_dil,4,-1,1);
Nuc_eros = berosion(Nuc_close,1,-1,1);
Nuc_eros(:,:,0)=0;
Nuc_eros(:,:,(ImstackSizeZ+1))=0;
filledmaskNuc=Nuc_eros;

for z=0:(ImstackSizeZ+1)
BinImageNuc=logical(Nuc_eros(:,:,z)); 
filledmaskNuc(:,:,z)= imfill(BinImageNuc,'holes');
end
Nuclab=label(filledmaskNuc,2,2000,0); % creates nuc label
NucMsr=measure(Nuclab,NuclM3D,({'Size'}),[],2,2000,0);  %measure nuc
[Nuctest,idee]=sort(NucMsr.Size,'descend');%sort nuc
Nucsize=NucMsr.Size;% calculates Nuc size
NucSegm=(Nuclab); % gives the image of the Nuc


if maxproj==1
writeim(uint16(max(MitoM3D,[],3)),[outdir,prename,'MitoM3D_MAX.tif'],'TIFF',1); % writes MitoM3D as tif
writeim(uint16(max(THF,[],3)),[outdir,prename,'MitoTHF_MAX.tif'],'TIFF',1); % writes MitoTHF as tif
writeim(uint16(max(Mitomask,[],3)),[outdir,prename,'MitoMask_MAX.tif'],'TIFF',1);% writes Mitomask as tif
writeim(uint16(max(Mitolab,[],3)),[outdir,prename,'Mitolab_MAX.tif'],'TIFF',1);% writes Mitolab as tif
end

if maxproj==1
 for z=2:(ImstackSizeZ+1)
imwrite(uint16(MitoM3D(:,:,z)),[outdir,prename,'MitoM3D.tif'], 'writemode', 'append'); % creates MaxProj of MitoM3D
imwrite(uint16(THF(:,:,z)),[outdir,prename,'MitoTHF.tif'], 'writemode', 'append');% creates MaxProj of MitoTHF
imwrite(uint16(Mitomask(:,:,z-1)),[outdir,prename,'MitoMask.tif'], 'writemode', 'append');% creates MaxProj of Mitomask
imwrite(uint16(Mitolab(:,:,z-1)),[outdir,prename,'Mitolab.tif'], 'writemode', 'append');% creates MaxProj of Mitolab
end
%end

NrMito=max(max(max(Mitolab))); %calculates the number of mitochondrial objects
MitoMsr=measure(Mitolab,MitoRAW,({'Size','Sum','Mean','Minimum','Maximum'}),[],2,0,0); %measures the mito
[mitotest,idee]=sort(MitoMsr.Size); 
[mitotestR,ideeR]=sort(MitoMsr.Size,'descend');

% 2D Mitochondria Measurements
toosmall=0;
mito_init=0;
clear MitoMsrComb
betterNuclmask=(calcmask*Nuclmask); %creates a beter nucleus mask
cytoplasm=(calcmask-betterNuclmask); %calculates cytoplasm mask
cytoplasmVol=sum(cytoplasm)*Vxlsz;% calculates cytoplasm volumr in 3D

 
% 2D Mitochondria Measurements
toosmall=0;
mito_init=0;
clear MitoMsrComb

for mito=1:NrMito
    mito;
    mito_mask=(Mitolab((MitoMsr(mito).Minimum(1)+0):(MitoMsr(mito).Maximum(1)+0),(MitoMsr(mito).Minimum(2)+0):(MitoMsr(mito).Maximum(2)+0),:))==mito;
    mito_proj=logical(max(mito_mask,[],3));
    mito_raw=uint16(MitoRAW((MitoMsr(mito).Minimum(2)+1):(MitoMsr(mito).Maximum(2)+1),(MitoMsr(mito).Minimum(1)+1):(MitoMsr(mito).Maximum(1)+1),:));
    mito_3d_raw=mito_mask*mito_raw;
    mito_rawproj=squeeze(max(mito_3d_raw,[],3));
    
if ndims(mito_rawproj)>1
MitoMsr2=measure(mito_proj,mito_rawproj,({'Size','Feret','AspectRatioFeret','Perimeter'}),[],2,0,0);
MitoMsr2.id=mito;

if mito_init==0
MitoMsrComb=MitoMsr2;
mito_init=1;
else 
MitoMsrComb=[MitoMsrComb;MitoMsr2];   
end


else 
toosmall=toosmall+1 ;
end


C{2,1}=prename; %sample name
C{2,2}=NrMito; %number of mitochondrial objects
C{2,6}=cytoplasmVol; %cytoplasm volume
C{2,7}=Nucvol;% nucleus volume

q=2;

C{1,1}='NameofFile';
C{1,2}='NumberofMitoObject';
C{1,3}='VolumeofMitoObject';
C{1,4}='MeanIntensityofMitoObject';
C{1,5}='SumIntensityofMitoObject';
C{1,6}='VolumeofCytoplasm';
C{1,7}='Volumeofnucleus';
 %mito

 C{q,3}=(MitoMsr(mito).Size)*Vxlsz; % writes individual mitochondrial object volume in a tif file
 C{q,4}=MitoMsr(mito).mean; %writes mean intensity of individual mitochondrial object volume in a tif file
 C{q,5}=MitoMsr(mito).sum; %writes sum intensity of individual mitochondrial object volume in a tif file
 C{q,6}=MitoMsrComb(mito).Feret(1,:);%writes feret of individual mitochondrial object volume in a tif file
 C{q,7}=cytoplasmVol; % writes the cytoplasm volume of cells in a tif file
    

    q=q+1; %%%
end %mito

 

xlswrite([outdir,prename,'_quant.xls'],C);

 

end


%%
